import audioop
from array import array
from struct import Struct

ADPCM_BLOCKSIZE = 36
PCM_BLOCKSIZE   = 130


def decode_mono_adpcm_samples(samples, swap_nibbles=True):
    adpcm2lin = audioop.adpcm2lin

    pcm_size   = PCM_BLOCKSIZE
    adpcm_size = ADPCM_BLOCKSIZE
    state_size = 4

    block_ct = len(samples) // adpcm_size
    out_data = bytearray(block_ct * pcm_size)

    pcm_i = 0
    unpacker = Struct("<hh").unpack_from
    if swap_nibbles:
        for i in range(0, len(samples), adpcm_size):
            # why couldn't it be nice and just follow the same
            # step packing pattern where the first step is the
            # first 4 bits and the second is the last 4 bits.
            steps = bytes(((b<<4) + (b>>4))&0xFF for b in
                          samples[i + state_size: i + adpcm_size])
            out_data[pcm_i: pcm_i + pcm_size] = (
                samples[i: i+2] + adpcm2lin(steps, 2, unpacker(samples, i))[0]
                )

            pcm_i += pcm_size
    else:
        for i in range(0, len(samples), adpcm_size):
            out_data[pcm_i: pcm_i + pcm_size] = (
                samples[i: i+2] +
                adpcm2lin(samples[i + state_size: i + adpcm_size],
                          2, unpacker(samples, i))[0]
                )
            pcm_i += pcm_size

    return array("h", out_data)


if __name__ == "__main__":
    header = b'RIFFXXXXWAVEfmt \x10\x00\x00\x00\
\x01\x00\x01\x00\x22\x56\x00\x00\x22\x56\x00\x00\x02\x00\x10\x00dataXXXX'
    with open("mono_adpcm_22050.wav", "rb")  as fi,\
         open("mono_16bit_pcm_22050_GOOD.wav", "wb+") as fog,\
         open("mono_16bit_pcm_22050_BAD.wav",  "wb+") as fob:
        fi.seek(48)
        adpcm_data = fi.read()
        good_pcm_data = decode_mono_adpcm_samples(adpcm_data)
        bad_pcm_data  = decode_mono_adpcm_samples(adpcm_data, False)

        for file, data in ((fog, good_pcm_data), (fob, bad_pcm_data)):
            data_len = len(data) * data.itemsize
            file.write(header)
            file.seek(4)
            file.write(int.to_bytes(len(header) + data_len - 12, 4, 'little'))
            file.seek(40)
            file.write(int.to_bytes(data_len, 4, 'little'))
            file.write(data)
